/////////////////////////////////////////////////////////////////////////////
// Serializable Object File Handler (Version 1)                            //
// FileHandler.java                                                        //
//                                                                         //
// Author: Devraj Mukherjee                                                //
// Last Modified: 19th September 2000                                      //
//                                                                         //
// This utility was developed in co-ordination with the espresso Servlet   //
// based web mail project. This is expected to function only when the      //
// the programs follow the class description.                              //
//                                                                         //
// Class function description:                                             //
// Instances of this class can write to or read from Serial JAVA Binary    //
// files, when the collection of objects have been stored in a Vector.     //
// A vector in the instances of the class can be set or retrieved allowing //
// changing the data as a collection only.                                 //
// Instances are capables of handling situations where the file does not   //
// exist. (New files are created)                                          //
// (See method reference for details)                                      //
//                                                                         //
// Method Reference:                                                       //
// FileHandler()       - Creates an instance with filename set to null     //
// FileHandler(String) - Creates an instance with the filename provided    //
// write()             - Writes to default file                            //
// write(String)       - Writes the vector to the specified file           //
// write(Vector)       - Writes the new vector to the default file         //
// read()              - Reads from default file                           //
// read(String)        - Reads from the specified file                     //
// setData(Vector)     - Replaces the current Vector with a new one        //
// getData()           - Returns a Vector with the Collection of objects   //
//                                                                         //
// Private Methods:                                                        //
// fileExists()        - Returns true/false if a file exists or doesn't    //
//                                                                         //
// Included in the FreeForums project on 22nd April 2001                   //
/////////////////////////////////////////////////////////////////////////////
package freeforums;

import java.io.*;
import java.util.Vector;

public class FileHandler implements Serializable {

 protected String filename;
 protected Vector dataVector;

 public FileHandler(String file_name, boolean readnow) throws IOException, ClassNotFoundException {
  filename=file_name;
  if (readnow) read();
 }

 public FileHandler() {
  filename=null;
 }

 public boolean write(String file_name) throws IOException, ClassNotFoundException {
  filename=file_name;
  return write();
 }

 public boolean write(Vector data, String file_name) throws IOException, ClassNotFoundException {
  setData(data);
  filename=file_name;
  return write();
 }

 public boolean write(Vector data) throws IOException, ClassNotFoundException {
  setData(data);
  return write();
 }

 public boolean write() throws IOException, ClassNotFoundException {
  if (filename==null) return false;
  ObjectOutputStream outfile=new ObjectOutputStream(new FileOutputStream(filename));
  outfile.writeObject(dataVector);
  outfile.close();
  return true;
 }

 public boolean read(String file_name) throws IOException, ClassNotFoundException {
  filename=file_name;
  return read();
 }

 public boolean read() throws IOException, ClassNotFoundException {
  if (filename==null) return false;
  if (fileExists()==false) return false; // check if the file exists

  ObjectInputStream infile=new ObjectInputStream(new FileInputStream(filename));
  dataVector=(Vector)infile.readObject();
  infile.close();
  return true;
 }

 public void setData(Vector data) {
  dataVector=data;
 }

 public Vector getData() {
  return dataVector;
 }

 private boolean fileExists() throws IOException, ClassNotFoundException {
  if (filename==null) return false;

  File dataFile=new File(filename);
  return dataFile.exists(); // return if the file exists or not
 }
 
}
