package freeforums;

import java.io.*;
import java.util.*;

public class MessageCollection extends Object implements Serializable {

 protected Vector messages;
 protected String filename;
 protected String configurationFile;

 // These two variables are used by the isRead method only
 // The values are given by the toHTML method
 protected String readmessagesFile;
 protected String UserName;

 // The variable is only used by the toHTML method
 protected int depth;
   
 public MessageCollection(String conf) {
  filename=null;
  configurationFile=conf;
  messages = new Vector();
 }

 public MessageCollection(String in_filename, String confFile) throws IOException, ClassNotFoundException {
  configurationFile=confFile;
  open(in_filename);
 }

 public void open(String file_name) {
  filename=file_name;
  FileHandler file=null;
  try {
   file=new FileHandler(filename,true);
  }
  catch(IOException e) {
   System.err.println("Error opening file");
  }
  catch(ClassNotFoundException e) {
   System.err.println("Class Not found in package");
  }
  messages=file.getData();
 }

 public void newFile() throws IOException, ClassNotFoundException {
  filename=null;
  messages = new Vector();
 }

 public String getFilename() { return filename; }

 public int getNumMessages() { return messages.size(); }

 public Message getMessage (int inMessageId) {
  for (int i=0; i<messages.size(); ++i)
   if (((Message)messages.elementAt(i)).getMessageId()==inMessageId)
    return (Message) messages.elementAt(i);
  return null;
 }

 public Message getMessageByIndex (int index) {
  return (Message) messages.elementAt(index);
 }

 public void putMessage (String inSubject, String inMessageText, String inUsername, int inReplyto, Date inDate) {
  ConfigurationReader reader=new ConfigurationReader(configurationFile);
  int OId=Integer.parseInt(reader.getValue("ObjectID"));
  messages.addElement (new Message(OId,inSubject,inMessageText,inUsername,inReplyto,inDate));
  OId++; //add one to object id
  reader.setValue("ObjectID",""+OId);
  reader.write();
 }
 
 public boolean deleteMessage (int inMessageId) {
  for (int i=0; i<messages.size(); ++i)
   if (((Message)messages.elementAt(i)).getMessageId()==inMessageId) {
    messages.removeElementAt(i);
    return true;
   }
  return false;
 }

 public void save() {
  FileHandler file=new FileHandler();
  try {
   file.write(messages,filename);
  }
  catch(IOException e) {
   System.err.println("Error opening file");
  }
  catch(ClassNotFoundException e) {
   System.err.println("Class Not found in package");
  }
 }

 public void saveAs (String new_filename) {
  filename = new_filename;
  save();
 }

 public String listAll() {
  String outString="";
  for(int cntr=0; cntr<messages.size(); cntr++) 
   outString+=(Message)messages.elementAt(cntr)+"\n";
  return outString;
 }

 public String toString() {
  String outString="";
  Message currentMessage;
  for(int cntr=0; cntr<messages.size(); cntr++) {
   currentMessage=(Message)messages.elementAt(cntr);
   if(currentMessage.getReplytoId()==-1) {
    outString=outString+currentMessage+"\n";
    outString=outString+listSubMessages(currentMessage.getMessageId());
   }
  }
  return outString;
 }

 private int countSubMessages(int MessageId) {
  int countMessages=0;
  for(int cntr=0; cntr<messages.size(); cntr++)
   if(((Message)messages.elementAt(cntr)).getReplytoId()==MessageId) countMessages++;
  return countMessages;
 }

 private String listSubMessages(int inMessageId) {
  Message currentMessage=null;
  String outString="";
  int countMessages=0,messageIndex=0;
  for(int cntr=0; cntr<messages.size(); cntr++) {
   currentMessage=(Message)messages.elementAt(cntr);
   if(currentMessage.getReplytoId()==inMessageId) {
    countMessages++;
    messageIndex=cntr;
   }
  }
  if (countMessages==0) return "";
  if (countMessages==1) {
   currentMessage=(Message)messages.elementAt(messageIndex);
   outString=currentMessage.toString()+"\n";
   if(countSubMessages(currentMessage.getMessageId())>0) outString+=listSubMessages(currentMessage.getMessageId());
   return outString;
  }
  ///////////////////////////////////
  // If more than one message then //
  ///////////////////////////////////
  for(int cntr=0; cntr<messages.size(); cntr++) {
   currentMessage=(Message)messages.elementAt(cntr);
   if(currentMessage.getReplytoId()==inMessageId) {
    outString=outString+currentMessage+"\n";
    // List sub messages for this reply message
    if(countSubMessages(currentMessage.getMessageId())>0)
     outString+=listSubMessages(currentMessage.getMessageId());
   }
  }
  ///////////////////////////////
  // End more than one message //
  ///////////////////////////////
  return outString;
 }

 ////////////////////////////
 // Methods for the web    //
 // above methods modified //
 // to produce HTML        //
 ////////////////////////////

 public String toHTML(String readFile, String inUserName) {
  readmessagesFile=readFile;
  UserName=inUserName;
  String outString="";
  Message currentMessage;
  for(int cntr=0; cntr<messages.size(); cntr++) {
   currentMessage=(Message)messages.elementAt(cntr);
   if(currentMessage.getReplytoId()==-1) {
    depth=0;
    outString=outString+currentMessage.toHTML(isRead(currentMessage.getMessageId()),depth)+"\n";
    outString=outString+listSubMessagesAsHtml(currentMessage.getMessageId());
   }
  }
  return outString;
 }

 private String listSubMessagesAsHtml(int inMessageId) {
  Message currentMessage=null;
  String outString="";
  int countMessages=0,messageIndex=0;
  depth+=8;
  for(int cntr=0; cntr<messages.size(); cntr++) {
   currentMessage=(Message)messages.elementAt(cntr);
   if(currentMessage.getReplytoId()==inMessageId) {
    countMessages++;
    messageIndex=cntr;
   }
  }
  if (countMessages==0) return "";
  if (countMessages==1) {
   currentMessage=(Message)messages.elementAt(messageIndex);
   outString+=currentMessage.toHTML(isRead(currentMessage.getMessageId()),depth)+"\n";
   if(countSubMessages(currentMessage.getMessageId())>0) outString+=listSubMessagesAsHtml(currentMessage.getMessageId());
   depth-=8;
   return outString;
  }
  ///////////////////////////////////
  // If more than one message then //
  ///////////////////////////////////
  for(int cntr=0; cntr<messages.size(); cntr++) {
   currentMessage=(Message)messages.elementAt(cntr);
   if(currentMessage.getReplytoId()==inMessageId) {
    outString=outString+currentMessage.toHTML(isRead(currentMessage.getMessageId()),depth)+"\n";
    // List sub messages for this reply message
    if(countSubMessages(currentMessage.getMessageId())>0)
     outString+=listSubMessagesAsHtml(currentMessage.getMessageId());
   }
  }
  ///////////////////////////////
  // End more than one message //
  ///////////////////////////////
  depth-=8;
  return outString;
 }

 private boolean isRead(int MessageId) {
  ReadMessageCollection readmessages=null;
  try {
   readmessages=new ReadMessageCollection(readmessagesFile);
  }
  catch(IOException e) {
   //do nothing
  }
  return readmessages.isRead(UserName,MessageId);
 }

} //End Class



